;***************************************************************************
; AT13  timer 1-1023 minutes (17 h), 0 = 1 min
;	ADC channel 1, potentiometer to set time
;	switch hi=minutes, lo=seconds (17 min)
;	pbStart switch Relais on , Relais off after time elapsed
; 	based on ATtiny15 GS 11-2009
;	The timing is adapted for 1.2 MHz 
;
;	lot of changes AT15 -> AT13
;	Interrupt vectors, TCCR0->TCCR0B, TIMSK0, TIFR0, GTCCR, ADCSRA
;	PortB PB4->PB3
;
;  This program is free software; you can redistribute it and/or
;  modify it under the terms of the GNU General Public License.
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; 
;
;***************************************************************************
;
;		Pinout ATtiny13/ATtiny13V   8-PDIP/SOIC
;
;(PCINT5/RESET/ADC0/dW)	PB5	VCC                             
;(PCINT3/CLKI/ADC3) 	PB3	PB2 (SCK/ADC1/T0/PCINT2)        
;(PCINT4/ADC2) 		PB4	PB1 (MISO/AIN1/OC0B/INT0/PCINT1)
;			GND	PB0 (MOSI/AIN0/OC0A/PCINT0)     
;
;***************************************************************************
; PB0	output	toggle pin (calibrate 10 ms hi or 50 Hz)
; PB1	input   switch hi=minutes, lo=seconds
; PB2	input   ADC channel 1 potentiometer to set time
; PB4	output  active high Relais
; PB3	input	start pushbutton (to lo)
;***************************************************************************

.DEVICE ATtiny13	;for gavrasm

.equ	toggle		= PB0 
.equ	swMinSec	= PB1 
;ADC1 = PB2
.equ	Relais		= PB4 
.equ	pbStart		= PB3 

;.equ	OSCCAL_val	= 0x6F			; calibrate OSCCAL or use c_value

.equ	c_value		= 189-1			; Compare Match 10 ms, 53 x 189
.equ 	cnt_cmp		= 100 			; 100 * 10 ms = 1 s, inc counter in TIM0_Cmp                              		

.def	S		= r4    		
.def 	lowByte 	= r5    		; ADC
.def 	highByte 	= r6    		                               		
.def	temp		= r16   		
.def 	counter		= r17   		
.def 	flag		= r20   		
.def	cntminuteL 	= r21			; main timing 
.def	cntminuteH 	= r22   		
.def	cnt60sec 	= r23   		
                                		
.cseg                           		
.org 0                          		
; Reset-vector to adress 0000  AT13 		

		rjmp	RESET			; Reset Handler
		reti				; IRQ0 Handler
		reti				; PCINT0 Handler
		reti				; Timer0 Overflow Handler
		reti				; EEPROM Ready Handler
		reti				; Analog Comparator Handler
		rjmp	TIM0_COMPA		; Timer0 CompareA Handler
		reti				; Timer0 CompareB Handler
		reti				; Watchdog Interrupt Handler
		reti				; ADC Conversion Handler

	
;********************************************************************
; TIM0_COMPA - Timer/counter 0 Compare MatchA interrupt handler
;
; interrupts every 10 ms, 53 x 189 
; 
;********************************************************************
TIM0_COMPA:	in	S,sreg			; Updated every 10 ms, calibrate OSCCAL

		sbis	PINB,  toggle		; skip if set
		sbi	PORTB, toggle		; output PORTB, toggle pin
		sbic	PINB,  toggle	 	; skip if set
		cbi	PORTB, toggle
		
		inc	counter			; used for main timing
TIM0_ex:	out	sreg,S
		reti
;***************************************************************************
reset:
		; writing calibration byte to the OSCCAL Register.
		;ldi	temp,OSCCAL_val 	; test 10 ms
		;out	OSCCAL,temp		; for the clock
		;nop
		;nop
;***************************************************************************
; Port B 
		ldi	temp,0b00001010 	; set pullup and pins
		out	PORTB,	temp		; 1 = pull-up , 0 = float
		ldi	temp,0b00010001 	; 1 = output , 0 = input
		out	DDRB,	temp       	; to data direction register

	
;***************************************************************************
; AD-Converter setup:
		ldi temp, 0b00000001  		; ADC1 input=PB2, REFS0,1 =0 Vcc is RefVoltage
		out ADMUX, temp			; 0..1023
						; ADCSRA: ADEN ADSC ADFR ADIF ADIE ADPS2 ADPS1 ADPS0	
		ldi temp,0b11010100 		; ADC initialise single conversion and prescaler 16:
		out ADCSRA, temp
;***************************************************************************
; timer0 setup:
						; Timer/Counter0 compare match interrupt Enable
		ldi	temp,(1<<WGM01)	 	; CTC1: Clear Timer/Counter on Compare Match
		out	TCCR0A,temp 		; 
		ldi	temp,0b00000100 	; 
		out	TIFR0,temp 		; clear pending Interrupt 
		out	TIMSK0,temp 		; set in the Timer Interrupt Mask Register
		ldi	temp,0			; Timer/Counter 0 clear
		out	TCNT0,temp		;
		ldi	temp,c_value		; Timer/Counter 0 for 10 ms
		out	OCR0A,temp		; 
		ldi	temp,0b00000011		; Timer/Counter 0 clocked at CK/64 = 53 us
		out	TCCR0B,temp		; @ 1,2 Mhz 
	
		sbi	AcsR,	Acd		; Disable Comparator                     

		cbi	PORTB, Relais
		sei				; Enable gobal iterrupt

;***************************************************************************
; timer time set with potentiometer (10k) on ADC1 1-1024 Minutes (17 h)
; read ADC1 at start
;***************************************************************************

loop0:  	ldi flag, 1			; remember swMinSec                        	
		sbic PINB, pbStart   		; skip if button is pressed = lo
		rjmp loop0              	; wait for start
		clr counter
		sbis PINB, swMinSec   		; skip if swMinSec is hi
		clr flag			; else if swMinSec is lo, seconds
		
debounce:	cpi counter, 2
		brne debounce			; debounce pbStart
		sbic PINB, pbStart   		; check again if button is pressed = lo
		rjmp loop0              	
        	                        	
		; get ADC value	        	
		sbi ADCSRA, ADIF 		; clear ADIF flag 
		sbi ADCSRA, ADSC  		; start conversion
wait1:  	         			; discard first reading
		sbis ADCSRA, ADIF        	
		rjmp wait1              	
        	                        	
		sbi ADCSRA, ADIF 		; clear ADIF flag 
		sbi ADCSRA, ADSC  		; start conversion
wait2:  	         			; read second ADC value
		sbis ADCSRA, ADIF        	
		rjmp wait2              	
						; ADC read:
		in lowByte, ADCL  		; low byte 
		in highByte, ADCH 		; then high byte
		                        	
		tst highByte			; no zeros
		brne loopset            	
		tst lowByte             	
		brne loopset			; ok, is <> 0
		inc lowByte			; 1 is minimum		                        	
		
loopset:                                	
						; Software-Counter-Register reset to zero
		ldi	counter,0 		; 10 ms
		ldi	cnt60sec, 0 		; 60 seconds
		ldi	cntminuteL,0 		; to zero L
		ldi	cntminuteH,0 		; to zero H
		
		ldi	temp,0     		; set register zero and ...
		out	TCNT0,temp 		; reset hardware-counter LSB
		ldi	temp,0b00000001  	; set Bit 0 PSR1: Prescaler Reset Timer/Counter0
		out	GTCCR,temp 		; Timer0 Prescaler Reset
		
		sbi	PORTB,Relais  		; hi out on Relais
loop1sec: 
		cpi 	counter, cnt_cmp	; 100 * 10 ms = 1 s, inc counter in TIM1_Cmp 
		brlo	loop1sec        			                        	
		ldi	counter, 0 		; back to 0 seconds

		tst	flag
		breq	loopSec 		; if 0 count seconds
			                        	                        	
		inc	cnt60sec		; inc 60 s counter		                        	
loop60:                                 	
		cpi 	cnt60sec, 60		; 1 s * 60 = 1 min
		brlo	loop1sec        	
		ldi	cnt60sec, 0 		; counter cnt60sec back to 0 minutes
loopSec:	                        	
		inc	cntminuteL		; inc main minute counter
		brne	niminh
		inc	cntminuteH
niminh:
 
		cp 	cntminuteH, highByte
		brlo	loop1sec
						; now wait for High byte, Minute vs ADC
		cp 	cntminuteL, lowByte
		brlo	loop1sec
		
		cbi	PORTB,Relais		; low Relais, end time loop

waithi:		sbis	PINB, pbStart   	; skip if button is hi again
		rjmp    waithi              	; else wait for hi
		rjmp	loop0			; when time elapsed and pbStart hi
